package com.budderfly.aws.gateway.service;

import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import software.amazon.awssdk.auth.credentials.AwsBasicCredentials;
import software.amazon.awssdk.auth.credentials.StaticCredentialsProvider;
import software.amazon.awssdk.core.SdkBytes;
import software.amazon.awssdk.core.client.config.ClientOverrideConfiguration;
import software.amazon.awssdk.core.internal.retry.SdkDefaultRetrySetting;
import software.amazon.awssdk.core.retry.RetryPolicy;
import software.amazon.awssdk.core.retry.backoff.BackoffStrategy;
import software.amazon.awssdk.core.retry.conditions.RetryCondition;
import software.amazon.awssdk.regions.Region;
import software.amazon.awssdk.services.firehose.FirehoseAsyncClient;
import software.amazon.awssdk.services.firehose.model.FirehoseResponse;
import software.amazon.awssdk.services.firehose.model.Record;

import java.util.ArrayList;
import java.util.List;
import java.util.function.BiConsumer;

public abstract class KinesisFirehoseService implements BiConsumer<FirehoseResponse, Throwable> {

    private final FirehoseAsyncClient firehoseAsyncClient;
    private final String streamName;
    Logger logger = LoggerFactory.getLogger(KinesisFirehoseService.class);
    ClientOverrideConfiguration configuration;

    public KinesisFirehoseService(String accessKey, String secretKey, String region, String streamName) {
        AwsBasicCredentials credentials = AwsBasicCredentials.create(accessKey, secretKey);
        configuration = ClientOverrideConfiguration.builder()
            .retryPolicy(RetryPolicy.builder()
                .backoffStrategy(BackoffStrategy.defaultStrategy())
                .numRetries(SdkDefaultRetrySetting.DEFAULT_MAX_RETRIES)
                .retryCondition(RetryCondition.defaultRetryCondition())
                .build())
            .build();
        firehoseAsyncClient = FirehoseAsyncClient.builder().credentialsProvider(StaticCredentialsProvider.create(credentials))
            .region(Region.of(region))
            .overrideConfiguration(configuration).build();
        this.streamName = streamName;
    }

    /**
     * Puts a record on the firehose stream
     *
     * @param streamName name of the delivery stream
     * @param record     string record to put on the delivery string
     */
    public void putRecord(String streamName, String record) {
        if (streamName != null && record != null && !record.isEmpty()) {
            logger.debug("entry putRecord for stream {}", streamName);
            SdkBytes data = SdkBytes.fromUtf8String(record + System.lineSeparator());
            firehoseAsyncClient.putRecord(builder ->
                builder
                    .deliveryStreamName(streamName)
                    .record(b -> b.data(data).build())
                    .build()).whenCompleteAsync(this);
        } else {
            logger.error("Kinesis Firehose not configured or no records to be send");
        }
    }

    /**
     * Puts a record on the firehose stream
     *
     * @param records    Arraylist of strings to put on the delivery batch
     */
    public void putRecordBatch(List<String> records) {
        if (streamName != null && records != null && !records.isEmpty()) {
            logger.debug("entry putRecordBatch {} nr of records {}", streamName, records.size());
            List<Record> recordList = new ArrayList<>();
            for (String record : records) {
                SdkBytes data = SdkBytes.fromUtf8String(record + System.lineSeparator());
                Record recordData = Record.builder().data(data).build();
                recordList.add(recordData);
            }
            firehoseAsyncClient.putRecordBatch(builder ->
                builder
                    .deliveryStreamName(streamName)
                    .records(recordList)
                    .build()).whenCompleteAsync(this);
        } else {
            logger.error("Kinesis Firehose not configured or no records to be send");
        }
    }
}
